/*******************************************************************************
 * This file is part of OpenNMS(R).
 *
 * Copyright (C) 2017-2017 The OpenNMS Group, Inc.
 * OpenNMS(R) is Copyright (C) 1999-2017 The OpenNMS Group, Inc.
 *
 * OpenNMS(R) is a registered trademark of The OpenNMS Group, Inc.
 *
 * OpenNMS(R) is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * OpenNMS(R) is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with OpenNMS(R).  If not, see:
 *      http://www.gnu.org/licenses/
 *
 * For more information contact:
 *     OpenNMS(R) Licensing <license@opennms.org>
 *     http://www.opennms.org/
 *     http://www.opennms.com/
 *******************************************************************************/

package org.opennms.netmgt.flows.classification.internal.matcher;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import org.opennms.netmgt.flows.classification.ClassificationRequest;
import org.opennms.netmgt.flows.classification.persistence.api.Protocol;
import org.opennms.netmgt.flows.classification.persistence.api.Protocols;
import org.opennms.netmgt.flows.classification.internal.value.StringValue;

public class ProtocolMatcher implements Matcher {

    private final List<Protocol> protocols = new ArrayList<>();

    public ProtocolMatcher(String protocols) {
        final StringValue stringValue = new StringValue(protocols);
        this.protocols.addAll(stringValue.splitBy(",")
                .stream()
                .map(p -> Protocols.getProtocol(p.getValue()))
                .filter(p -> p != null)
                .collect(Collectors.toList()));
    }

    public ProtocolMatcher(int protocol) {
        this(Protocols.getProtocol(protocol));
    }

    public ProtocolMatcher(Protocol protocol) {
        this.protocols.add(Objects.requireNonNull(protocol));
    }

    @Override
    public boolean matches(ClassificationRequest request) {
        return protocols
                .stream()
                .filter(p -> p.getDecimal() == request.getProtocol().getDecimal())
                .findAny()
                .isPresent();
    }
}
